package websocket

import (
	"github.com/charmbracelet/log"
	"github.com/gorilla/websocket"
	"net"
	"net/http"
	"sync"

	"git.smsvc.net/pomodoro/GoTomato/pkg/models"
)

// Clients is a map of connected WebSocket clients, where each client is represented by the WebsocketClient struct
var Clients = make(map[net.Addr]*models.WebsocketClient)
var mu sync.Mutex // Mutex to protect access to the Clients map

// Upgrade HTTP requests to WebSocket connections
var upgrader = websocket.Upgrader{
	CheckOrigin: func(r *http.Request) bool { return true },
}

// Upgrades HTTP requests to WebSocket connections and manages the client lifecycle
func HandleConnection(w http.ResponseWriter, r *http.Request) {
	// Upgrade initial GET request to a WebSocket
	ws, err := upgrader.Upgrade(w, r, nil)
	if err != nil {
		log.Error("WebSocket upgrade error:", "msg", err)
		return
	}
	defer ws.Close()

	log.Info("Client connected", "host", ws.NetConn().RemoteAddr(), "clients", len(Clients)+1)

	// Register the new client
	client := models.WebsocketClient{
		Conn: ws,
	}
	mu.Lock()
	Clients[ws.LocalAddr()] = &client
	mu.Unlock()

	// Listen for commands from the connected client
	handleClientCommands(client)
}
